/* blssl/SSLConf.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */



#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <openssl/conf.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#include "com_brm_net_ssl_SSLConf.h"
#include "brm_ssl.h"
#include "brm_util.h"

/*
 * Class:     com_brm_net_ssl_SSLConf
 * Method:    nativeGetSSLConf
 * Signature: (Lcom/brm/net/ssl/SSLConf;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_brm_net_ssl_SSLConf_nativeGetSSLConf
  (JNIEnv *env, jobject obj, jstring confFile)
{
	char *fname;
	LHASH *conf = NULL;
	long eline = 0;
	const char *file;
	jboolean isCopy;
	BIO *bio = NULL;
	
	SSL_load_error_strings();
	bio = BIO_new(BIO_s_mem());

	file = (*env)->GetStringUTFChars(env, confFile, &isCopy);
	if(!file || file[0] == 0) 
	{
		BIO_printf(bio, "brm : unable to read confFile string\n");
		goto err;
	} 
	conf = CONF_load(NULL, file, &eline);
	if(isCopy == JNI_TRUE) 
		(*env)->ReleaseStringUTFChars(env, confFile, file);

	if(!conf) {
		ERR_print_errors(bio);
		BIO_printf(bio, 
			"brm : unable to load configuration, line %ld\n", eline);
		goto err;
	}

	fname = "SSLMessageTrace"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
/*	
	fname = "SSLSessionCache"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
*/
	fname = "SSLCipherSuite"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
	fname = "SSLCACertificatePath"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
	fname = "SSLCARevocationPath"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
	fname = "SSLCARevocationAutoUpdate"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
	fname = "SSLCertificateFile"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
	fname = "SSLCertificateKeyFile"; 
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));
	
	fname = "SSLHandShakeTimeout"; 
	set_jint(env, obj, fname, CONF_get_number(conf, NULL, fname));
	fname = "SSLSessionCacheTimeout"; 
	set_jint(env, obj, fname, CONF_get_number(conf, NULL, fname));

	fname = "SSLVerify";
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));

	fname = "SSLVerifyDepth";
	set_jint(env, obj, fname, CONF_get_number(conf, NULL, fname)); 

	fname = "SSLProtocol";
	set_jstr(env, obj, fname, CONF_get_string(conf, NULL, fname));

	CONF_free(conf);
	BIO_free(bio);
	return;
err:
	if(conf) CONF_free(conf);
	raise_exception(env, bio);
	if(bio) BIO_free(bio);
	return;
}



