/* blssl/SSLOutputStream.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */



#include <openssl/ssl.h>
#include <openssl/err.h>

#include "com_brm_net_ssl_SSLOutputStream.h"
#include "brm_ssl.h"
#include "brm_util.h"
#include "brm_socket.h"


/*
 * Class:     com_brm_net_ssl_SSLOutputStream
 * Method:    nativeWriteArray
 * Signature: (J[BII)[B
 */
JNIEXPORT jbyteArray JNICALL 
Java_com_brm_net_ssl_SSLOutputStream_nativeWriteArray
  (JNIEnv *env, 
  jobject obj, 
  jlong in, 
  jbyteArray indata, 
  jint inoff, 
  jint inlen)
{
	jbyteArray ret = NULL;
	unsigned char *data;
	int len, n;
	BRM_SSL *bssl = (BRM_SSL*)in;

	if(inlen > 0) { 
		data = (unsigned char*)OPENSSL_malloc(inlen);
		if(!data) {
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, 
				"brm : unable to allocate the memory size %d bytes", inlen);
			goto err;
		}
		(*env)->GetByteArrayRegion(env, indata, inoff, inlen, data);
		n = SSL_write(bssl->ssl, data, inlen);
		switch(SSL_get_error(bssl->ssl, n)) {
		case SSL_ERROR_NONE:
		case SSL_ERROR_WANT_WRITE:
		case SSL_ERROR_WANT_READ:
			break;
		case SSL_ERROR_ZERO_RETURN:
			OPENSSL_free(data);
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, "brm : SSL connection closed");
			goto err;
			break;
		/* Ʒ   ۿ    */
		case SSL_ERROR_SYSCALL:
		case SSL_ERROR_SSL:
		default:
			OPENSSL_free(data);
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, "brm : meets internal errors");
			goto err;
			break; 
		}

		BIO_flush(bssl->wbio);
		OPENSSL_free(data);
	}
	
	len = BIO_ctrl_pending(bssl->wbio);
	if(len) {
		data = (unsigned char*)OPENSSL_malloc(len);
		if(data == NULL) {
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, 
				"brm : unable to allocate the memory size %d bytes", len);
			goto err;
		} 
		ret = (*env)->NewByteArray(env, len);
		if(!ret) {
			OPENSSL_free(data);
			BIO_printf(bssl->errbio, 
				"brm : unable to NewByteArray the memory size %d bytes", len);
			goto err;
		}
		n = BIO_read(bssl->wbio, data, len);
		(*env)->SetByteArrayRegion(env, ret, 0, len, data);
		OPENSSL_free(data);
	} 
	return ret;
err:
	raise_exception(env, bssl->errbio);
	return NULL;
}



