/* blssl/SSLServer.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */



#include <stdio.h>
#include <string.h>

#ifdef WIN32
#include  <winsock.h>
#else
#include <sys/types.h>   /* for select */
#include <sys/time.h>
#endif

#include <openssl/ssl.h>
#include <openssl/err.h>

#include "com_brm_net_ssl_SSLServerSocket.h"
#include "blssl.h"
#include "brm_ssl.h"
#include "brm_util.h"
#include "brm_socket.h"


/*
 * Class:     com_brm_net_ssl_SSLServerSocket
 * Method:    nativeNew
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_com_brm_net_ssl_SSLServerSocket_nativeNew
  (JNIEnv *env, jobject obj, jlong in)
{
	BRM_SSL *bssl = NULL;
	BRM_SSL_CTX *bctx;
	
	bctx = (BRM_SSL_CTX *)in;
	bssl = (BRM_SSL*)OPENSSL_malloc(sizeof(BRM_SSL));
	if(!bssl) {
		fprintf(stderr,
			"brm : unable to allocate the memory size %d bytes", 
			sizeof(BRM_SSL));
		raise_exception(env, NULL);
		return 0;
	}
	memset(bssl, 0, sizeof(BRM_SSL));
	bssl->ssl = SSL_new(bctx->ctx); 
	if(!bssl->ssl) {
		fprintf(stderr, "brm : unable to allocate the ssl context");
		raise_exception(env, NULL);
		BRM_SSL_Free(bssl);
		return 0;
	}
	bssl->errbio = BIO_new(BIO_s_mem());
	if(!bssl->errbio) {
		fprintf(stderr, "brm : unable to allocate the bio io");
		raise_exception(env, NULL);
		BRM_SSL_Free(bssl);
		return 0;
	}
	bssl->wbio = BIO_new(BIO_s_mem());
	if(!bssl->wbio) {
		ERR_print_errors(bssl->errbio);
		BIO_printf(bssl->errbio, "brm : unable to allocate the bio io");
		goto err;
	}
	bssl->rbio = BIO_new(BIO_s_mem());
	if(!bssl->rbio) {
		ERR_print_errors(bssl->errbio);
		BIO_printf(bssl->errbio, "brm : unable to allocate the bio io");
		goto err;
	}
	SSL_set_bio(bssl->ssl, bssl->rbio, bssl->wbio);
	if(bctx->SSLMessageTrace) {
		bssl->SSLMessageTrace = bctx->SSLMessageTrace;
		BIO_set_callback(SSL_get_rbio(bssl->ssl), bio_dump_rcb);
		BIO_set_callback(SSL_get_wbio(bssl->ssl), bio_dump_wcb);
		bssl->trcbio = BIO_new_fp(stderr, BIO_NOCLOSE);
		BIO_set_callback_arg(SSL_get_rbio(bssl->ssl), bssl->trcbio);
		BIO_set_callback_arg(SSL_get_wbio(bssl->ssl), bssl->trcbio);
	}

	SSL_set_accept_state(bssl->ssl);

	bssl->bctx = bctx; 

	if(app_index < 0) {
		app_index = SSL_get_ex_new_index(0, app_index_name, NULL, NULL, NULL);
		/* OpenSSL  ø̼ ͸ ssl context 
		  ִ  ϴµ  ʹ ε еȴ. 
		׷Ƿ ϳ Ϳ ϳ ε ǵ ο ε 
		 Ѵ.  ؽ 鶧 ε  Լ ȣϸ ο 
		ε ϳ    . 
		 ε  Լ ȣϸ ε  0 Ѿµ 
		Ʒó ѹ  Լ ȣ ָ ε  1 ´. 
		̷  mod_ssl Ǿ mod_ssl ΰ ø̼ 
		͸ Ѵ. 

		app_index = SSL_get_ex_new_index(0, app_index_name, NULL, NULL, NULL);
		*/
	}
	/*
		 : Ʒ Լ   ٲ    ̴.
		#define SSL_set_app_data(s,arg)		(SSL_set_ex_data(s,0,(char *)arg))
		#define SSL_get_app_data(s)		(SSL_get_ex_data(s,0))
		SSL_set_app_data(bssl->ctx, ssl);
	*/
	SSL_set_ex_data(bssl->ssl, app_index, bssl);

	return (jlong)bssl;
err:
	raise_exception(env, bssl->errbio);
	if(bssl) BRM_SSL_Free(bssl);
	return 0;
}


/*
 * Class:     com_brm_net_ssl_SSLServerSocket
 * Method:    nativeAccept
 * Signature: (J[BII)[B
 */
JNIEXPORT jbyteArray JNICALL Java_com_brm_net_ssl_SSLServerSocket_nativeAccept
   (JNIEnv *env, 
   jobject obj, 
   jlong in, 
   jbyteArray indata, 
   jint inoff, 
   jint inlen)
{
	jbyteArray ret = NULL;
	BRM_SSL *bssl = (BRM_SSL*)in;
	unsigned char *data;
	unsigned char *buf = NULL;
	int len, n;

	if(inlen > 0) { 
		data = (unsigned char*)OPENSSL_malloc(inlen);
		if(data == NULL) {
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, 
				"brm : unable to allocate the memory size %d bytes", inlen);
			goto err;
		}
		(*env)->GetByteArrayRegion(env, indata, inoff, inlen, data);
		n = BIO_write(bssl->rbio, data, inlen);
		OPENSSL_free(data);
	}

	n = SSL_accept(bssl->ssl);
	if(1 != n) {
		switch(SSL_get_error(bssl->ssl, n)) {
			/*  */
			case SSL_ERROR_NONE:
			case SSL_ERROR_WANT_READ:
			case SSL_ERROR_WANT_WRITE:
			case SSL_ERROR_WANT_X509_LOOKUP:
			break;
		default:
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, 
				"brm : ssl server fails to process ssl handshake");
			goto err;
			break; 
		}
	}
 
	BIO_flush(bssl->wbio);
	len = BIO_ctrl_pending(bssl->wbio);
	if(len) {
		buf = (unsigned char*)OPENSSL_malloc(len);
		if(!buf) {
			ERR_print_errors(bssl->errbio);
			BIO_printf(bssl->errbio, 
				"brm : unable to malloc the memory size %d bytes", len);
			goto err;
		}
		ret = (*env)->NewByteArray(env, len);
		if(!ret) {
			OPENSSL_free(buf);
			BIO_printf(bssl->errbio, 
				"brm : unable to NewByteArray the memory size %d bytes", len);
			goto err;
		}
		n = BIO_read(bssl->wbio, buf, len);
		(*env)->SetByteArrayRegion(env, ret, 0, len, buf);
		OPENSSL_free(buf);
	} 
	return ret;
err:
	raise_exception(env, bssl->errbio);
	BIO_reset(bssl->rbio);
	BIO_reset(bssl->wbio);
	return NULL;
}


/*
 * Class:     com_brm_net_ssl_SSLServerSocket
 * Method:    nativeAcceptFinished
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL 
Java_com_brm_net_ssl_SSLServerSocket_nativeAcceptFinished
   (JNIEnv *env, jobject obj, jlong in)
{
	BRM_SSL *bssl = (BRM_SSL*)in;
	int ret;
	ret = handshake_finished(bssl);
	switch(ret) {
	case 1 : 
		return BRM_TRUE;
		break;
	case 0 :
		return BRM_FALSE;
		break;
	}
	raise_exception(env, bssl->errbio);
	return BRM_FALSE;
}


/*
 * Class:     com_brm_net_ssl_SSLServerSocket
 * Method:    nativeFree
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_brm_net_ssl_SSLServerSocket_nativeFree
 (JNIEnv *env, jobject obj, jlong in)
{
	BRM_SSL *bssl = (BRM_SSL*)in;

	if(bssl) BRM_SSL_Free(bssl);
}



