/* blssl/SSLSession.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */



#include <openssl/ssl.h>
#include <openssl/err.h>

#include "com_brm_net_ssl_SSLSession.h"
#include "brm_ssl.h"
#include "brm_util.h"


/*
 * Class:     com_brm_net_ssl_SSLSession
 * Method:    nativeNewSession
 * Signature: (J)[B
 */
JNIEXPORT jbyteArray JNICALL 
Java_com_brm_net_ssl_SSLSession_nativeNewSession
  (JNIEnv *env, jobject obj, jlong in)
{
	jbyteArray ret = NULL;
	BRM_SSL *bssl = (BRM_SSL *)in;
	unsigned char *p, *dsess = NULL;
	int len;

	len = i2d_SSL_SESSION(SSL_get_session(bssl->ssl), NULL);
	if(len == 0) {
		/*  б  */
		ERR_print_errors(bssl->errbio);
		BIO_printf(bssl->errbio, 
			"brm : unable to get the SSL session size");
		goto err;
	}
	p = dsess = OPENSSL_malloc(len);
	if(!dsess) {
		ERR_print_errors(bssl->errbio);
		BIO_printf(bssl->errbio, 
			"brm : unable to allocate the memory size %d bytes", len);
		goto err;
	}

	len = i2d_SSL_SESSION(SSL_get_session(bssl->ssl), &p);
	if(len == 0) {
		/*  б  */
		OPENSSL_free(dsess);
		ERR_print_errors(bssl->errbio);
		BIO_printf(bssl->errbio, 
			"brm : unable to get the SSL session");
		goto err;
	}
	ret = (*env)->NewByteArray(env, len);
	if(!ret) {
		OPENSSL_free(dsess);
		BIO_printf(bssl->errbio, 
			"brm : unable to NewByteArray the memory size %d bytes", len);
		goto err;
	}
	
	(*env)->SetByteArrayRegion(env, ret, 0, len, dsess);
	OPENSSL_free(dsess);
	return ret;
err:
	raise_exception(env, bssl->errbio);
	return ret;
}


