/* blssl/blssl.h */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#ifndef __BLSSL_H
#define __BLSSL_H

#ifdef BLSSL_INCLUDE_OPENSSL
#include <openssl/bio.h>
#include <openssl/ssl.h>
#else
#define SSL_CTX		void
#define X509_STORE	void 
#define SSL			void 
#define BIO			void
#define LHASH		void
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define BRM_OK							0
#define BRM_ERR_WRONG_INPUT				3000
#define BRM_ERR_OUTOF_MEMORY			3001
#define BRM_ERR_READ_CONF				3002
#define BRM_ERR_NO_FIELD				3003
#define BRM_ERR_FAILTO_SET_CIPHER		3004
#define BRM_ERR_LOAD_VERIFY_LOC			3005	
#define BRM_ERR_FAILTO_CREATE_CRL_STORE	3006
#define BRM_FAILTO_READ_CERT			3007
#define BRM_FAILTO_READ_PRIVATEKEY		3008
#define BRM_FAILTO_SSLCONNECT			3009
#define BRM_FAILTO_SSLACCEPT			3010
#define BRM_FAILTO_ASSIGNSESSION		3011


typedef struct global_context {
	SSL_CTX *ctx;
	int SSLHandShakeTimeout;
	long SSLSessionCacheTimeout;
	X509_STORE *RevocationStore;
	int SSLCARevocationAutoUpdate;
	int SSLMessageTrace;
	/* 
	char *SSLSessionCache;
	char *SSLEngine;
	*/
} BRM_SSL_CTX;

typedef struct ssl_context {
	BRM_SSL_CTX *bctx;
	SSL *ssl;
	BIO *wbio;
	BIO *rbio;
	BIO *appbio;
	BIO *errbio;
	int SSLMessageTrace;
	BIO *trcbio;
	int sock;
	int SSLHandShakeTimeout;
} BRM_SSL;

typedef struct brm_conf {
	LHASH *conf;
	int SSLHandShakeTimeout;
	int SSLSessionCacheTimeout;
	char *SSLCACertificatePath;
	char *SSLCARevocationPath;
	char *SSLCARevocationAutoUpdate;
	char *SSLVerify;
	int SSLVerifyDepth;
	char *SSLCertificateFile;
	char *SSLCertificateKeyFile;
	char *SSLProtocol;
	char *SSLCipherSuite;
	char *SSLMessageTrace;
} BRM_CONF;


void BRM_SSL_CTX_Free(BRM_SSL_CTX *bctx);
void BRM_SSL_Free(BRM_SSL *bssl);

#ifdef __cplusplus
}
#endif

#endif /* __BLSSL_H */



