/* blssl/brm_sock.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */


#include "brm_socket.h"

int SocketReadN(int fd, char* ptr, int nbytes)
{
	int nleft, nread;

	nleft = nbytes;
	while(nleft > 0)
	{
		nread = SOCK_READ(fd, ptr, nleft);
		if(nread < 0)
			return nread;  /* error, return < 0 */
		else if(nread == 0)
			break;			/* EOF */
		nleft -= nread;
		ptr += nread;
	}
	return (nbytes - nleft);   /* return >= 0 */
}

int SocketWriteN(int fd, char *ptr, int nbytes)
{
	int nleft, nwritten;

	nleft = nbytes;
	while(nleft > 0)
	{
		nwritten = SOCK_WRITE(fd, ptr, nleft);
		if(nwritten <= 0)
			return nwritten;		/* error */
		nleft -= nwritten;
		ptr += nwritten;
	}
	return (nbytes - nleft);
}
