/* blssl/brm_ssl.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */



#if defined(WIN32) || defined(_WIN32) || defined (WINDOWS) || defined(_WINDOWS)
#include <windows.h>
#endif

#include <string.h>

#include <openssl/bio.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>

#include "blssl.h"
#include "brm_ssl.h"

static int _verify_crl(int ok, 
					   X509_STORE_CTX *ctx, 
					   X509_STORE *store, 
					   int au, 
					   BIO *errbio);
static int _chk_crl_expired(X509_CRL *crl);

int app_index = -1;
char *app_index_name = "brm global index";


void BRM_SSL_CTX_Free(BRM_SSL_CTX *bctx)
{
	if(!bctx) return;
	if(bctx->RevocationStore) 
		X509_STORE_free(bctx->RevocationStore);
	if(bctx->ctx) 
		SSL_CTX_free(bctx->ctx);
	OPENSSL_free(bctx);
}


void BRM_SSL_Free(BRM_SSL *bssl)
{
	if(bssl) {
		if(bssl->ssl) SSL_free(bssl->ssl);
		/* Ʒ rbio, wbio  SSL_free() Լ ο ڵ ȣ  
		if(bssl->rbio) BIO_free(bssl->rbio);
		if(bssl->wbio) BIO_free(bssl->wbio); */
		if(bssl->appbio) BIO_free(bssl->appbio);
		if(bssl->errbio) BIO_free(bssl->errbio);
		if(bssl->trcbio) BIO_free(bssl->trcbio);
		memset(bssl, 0, sizeof(BRM_SSL));
		OPENSSL_free(bssl);
	}
}


/* -------------------------------------------------------------------------
 Լ̸ : handshake_finished
      : 
 Է     : [in] BRM_SSL *bssl
      : int 1 - handshake has finished
				0 - handshake has not finished yet
			   -1 - fail to allocate new application bio
------------------------------------------------------------------------- */
int handshake_finished(BRM_SSL *bssl)
{
	if(!SSL_is_init_finished(bssl->ssl))
		return 0; 
	
	if(bssl->SSLMessageTrace) 
		print_connect_info(bssl);

	if(bssl->appbio) 
		return 1;

	bssl->appbio = BIO_new(BIO_s_mem());
	if(!bssl->appbio) {
		ERR_print_errors(bssl->errbio);
		BIO_printf(bssl->errbio, 
			"brm : unable to allocate the application bio");
		return -1; 
	}
	return 1;
}


int cert_verify_cb(int ok, X509_STORE_CTX *ctx)
{
	int err,depth;
	SSL *ssl;
	BRM_SSL_CTX *bctx;
	BRM_SSL *bssl;
	int au;
	X509_STORE *store;
	BIO *bio;

	ssl = X509_STORE_CTX_get_ex_data(
		ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
	/* bctx = (BRM_SSL*)X509_STORE_CTX_get_app_data(ctx); */
    bssl = SSL_get_ex_data(ssl, app_index);
	bctx =  bssl->bctx;

	if(ok == 0) {
		err = X509_STORE_CTX_get_error(ctx);
		depth =	X509_STORE_CTX_get_error_depth(ctx);
		/*  OpenSSL ⺻   ߴٸ */
		/* 
		ssl/ssl_cert.c Ͽ 
		int ssl_verify_cert_chain(SSL *s,STACK_OF(X509) *sk) Լ  
		Ʒó Ǿ ִ. 
			...
			if (s->server)
				i = X509_PURPOSE_SSL_CLIENT;
			else
				i = X509_PURPOSE_SSL_SERVER;

			X509_STORE_CTX_purpose_inherit(&ctx, i, s->purpose, s->trust);
			...

		 SSL handshake ߿    ݵ ΰ  
		Ѱ purpose   õϵ Ǿ ִ. 
		׷Ƿ   purpose  ϴ  Ұϴ. */
		if(SSL_get_verify_depth(ssl) >= depth) {
			ok=1;
			ctx->error = X509_V_OK;
		} else {
			ok=0;
			ctx->error = X509_V_ERR_CERT_CHAIN_TOO_LONG;
		}
		/*    ִ  Ѵ. */
		switch(err) {
			/*  츮   뵵 Ȯϰ  
			ʾ OpenSSL   X509_V_ERR_INVALID_PURPOSE  
			߻Ų. ׷Ƿ   ߻ϸ Ѵ. */
		case X509_V_ERR_INVALID_PURPOSE:
			ok = 1;
			ctx->error = X509_V_OK;
			break;
		default:
			BIO_printf(bssl->errbio, 
				"\nverify error:%d:%s\n", 
				err, X509_verify_cert_error_string(err));
			return ok;
			break;
		}
	}
#if 0
	/*	by cha 2002-09-24 
	The crl check is not debuged yet.  
	It will be verified if it is needed in the near future. */
	if(bctx->RevocationStore) { 
		au = bctx->SSLCARevocationAutoUpdate;
		store = bctx->RevocationStore;
		bio = bssl->errbio;
		ok = _verify_crl(ok, ctx, store,  au, bio);
		/* BIO_printf(bio_err,"verify return:%d\n",ok); */
	}
#endif

    return ok;
}



/* -------------------------------------------------------------------------
 Լ̸ : _verify_crl
      : 
 Է     : [in] int ok
 Է     : [in] X509_STORE_CTX *ctx
 Է     : [in] X509_STORE *store
 Է     : [in] int au
 Է     : [in] BIO *errbio
      : int 
      : 
------------------------------------------------------------------------- */
int _verify_crl(int ok, 
				X509_STORE_CTX *ctx, 
				X509_STORE *store, 
				int au, 
				BIO *errbio)
{
	X509_OBJECT objx, objc;
    X509_NAME *subject;
    X509_NAME *issuer;
    X509 *xs, *xi;
    X509_CRL *crl;
    X509_REVOKED *revoked;
    ASN1_INTEGER *serial;
    BIO *bio;
    int i, n, len, ret;
    char *cp;
	int fetch = 0;
	X509_EXTENSION *ex;
	ASN1_OBJECT *obj;
	char *ldap_url = NULL;


#define FETCH_RETRY_COUNT		1

    /*
     * Unless a revocation store for CRLs was created we
     * cannot do any CRL-based verification, of course.
     */

    if(!store) return ok;

    xs      = X509_STORE_CTX_get_current_cert(ctx);
    subject = X509_get_subject_name(xs);
    issuer  = X509_get_issuer_name(xs);

retry:
	/* CRL  Ⱑ ̹ õ Ǿ   ̷ Ķ */
	if(fetch > 1) return ok;

	memset(&objx, 0, sizeof(objx));
	memset(&objc, 0, sizeof(objc));

	ret = X509_STORE_get_by_subject(ctx, X509_LU_X509, issuer, &objx);
	xi = objx.data.x509;
	if(ret < 1 || xi == NULL) { 
		cp = X509_NAME_oneline(X509_get_issuer_name(xs),  NULL, 0);
		BIO_printf(errbio, 
			"The issuer certificate [%s] is not found "
			"in the local cert store.\n", cp);
		if(cp) free(cp);
		X509_STORE_CTX_set_error(ctx, X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT);
		goto err;
	}

    /*
     * Try to retrieve a CRL corresponding to the _subject_ of
     * the current certificate in order to verify it's integrity.
     */
    
	ret = crl_lookup(store, issuer, &objc);
    crl = objc.data.crl;

    if (ret > 0 && crl != NULL) {
	    /* Verify the signature on this CRL */
        if (X509_CRL_verify(crl, X509_get_pubkey(xi)) <= 0) {
            X509_STORE_CTX_set_error(ctx, X509_V_ERR_CRL_SIGNATURE_FAILURE);
            goto err;
        }

		ret = _chk_crl_expired(crl);
		if(ret == X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD) {
            X509_STORE_CTX_set_error(ctx, X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD);
            goto err;
        }
        if(ret == X509_V_ERR_CRL_HAS_EXPIRED) {
			/*      : 
			 LDAPκ ̹ ο CRL Դٸ */
			if(fetch++ >= FETCH_RETRY_COUNT || !au) {
				X509_STORE_CTX_set_error(ctx, X509_V_ERR_CRL_HAS_EXPIRED);
				goto err;
			}
			/* ⿡  CRL  а ο CRL  
			õѴ. */
			/* ... */
			/*  CRL  д´.*/
			/* CRL  CRL  ´. */
			/* ϵ ũ Ѵ. (̶ ϸ  Ŵ  ?)*/
			X509_OBJECT_free_contents(&objx);
			X509_OBJECT_free_contents(&objc);
			goto retry;		
        }
    } else {
		
		if(!au) {
			cp = X509_NAME_oneline(X509_get_issuer_name(xs),  NULL, 0);
			BIO_printf(errbio, "The crl of issuer : [%s] is not found "
				"in the local crl store.\n", cp);
			if(cp) free(cp);
			X509_STORE_CTX_set_error(ctx, X509_V_ERR_UNABLE_TO_GET_CRL);
			goto err;
		}
		/* ⿡  CRL  а ο CRL õ */
		n = X509_get_ext_count(xs);
		if(n == 0) {
			cp = X509_NAME_oneline(X509_get_subject_name(xs),  NULL, 0);
			BIO_printf(errbio, 
				"The certificate [%s] has no X509 V3 extension field.\n", cp);
			if(cp) free(cp);
			X509_STORE_CTX_set_error(ctx, X509_V_ERR_UNABLE_TO_GET_CRL);
			goto err;
		}

		for(i = 0; i < n; i++) {
			ex = X509_get_ext(xs, i);
			obj = X509_EXTENSION_get_object(ex);
			if(NID_crl_distribution_points == OBJ_obj2nid(obj)) {
				bio = BIO_new(BIO_s_mem());
				/* ġȯ  ڿ  õ */
				if(!X509V3_EXT_print(bio, ex, 0, 0)) {
					/* ġȯ   ڿ̶ RAW   */
					M_ASN1_OCTET_STRING_print(bio, ex->value);
				}
				len = BIO_ctrl_pending(bio);
				ldap_url = OPENSSL_malloc(len + 1);
				BIO_read(bio, ldap_url, len);
				ldap_url[len] = 0;
				BIO_free(bio);
				break;
			}
		}
		/*   ... */
		/*   ϸ Ǵ   Ⱓ  CRL̸ */
		X509_STORE_CTX_set_error(ctx, X509_V_ERR_UNABLE_TO_GET_CRL);
		goto err;
		/* ϸ ... */
		goto retry;
	}

    n = sk_X509_REVOKED_num(X509_CRL_get_REVOKED(crl));
	serial = X509_get_serialNumber(xs);
    for (i = 0; i < n; i++) {
        revoked = sk_X509_REVOKED_value(X509_CRL_get_REVOKED(crl), i);
        if (ASN1_INTEGER_cmp(revoked->serialNumber, serial) == 0) {
            X509_STORE_CTX_set_error(ctx, X509_V_ERR_CERT_REVOKED);
            goto err;
        }
    }
	if(ldap_url) OPENSSL_free(ldap_url);
	X509_OBJECT_free_contents(&objx);
	X509_OBJECT_free_contents(&objc);
	return 1;
err:
	if(ldap_url) OPENSSL_free(ldap_url);
	X509_OBJECT_free_contents(&objx);
	X509_OBJECT_free_contents(&objc);
	return 0;
}


int _chk_crl_expired(X509_CRL *crl)
{
	int i;

	i = X509_cmp_current_time(X509_CRL_get_nextUpdate(crl));
    if (i == 0) return X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD;
	if(i < 0) return X509_V_ERR_CRL_HAS_EXPIRED;
	return X509_V_OK;
}

/* -------------------------------------------------------------------------
 Լ̸ : crl_lookup
      : 
 Է     : [in] X509_STORE *store
 Է     : [in] X509_NAME *name
 Է     : [in] X509_OBJECT *obj
      : int 
      : 
------------------------------------------------------------------------- */
int crl_lookup(X509_STORE *store, X509_NAME *name, X509_OBJECT *obj)
{
    int ret;
	X509_STORE_CTX ctx;

    X509_STORE_CTX_init(&ctx, store, NULL, NULL);
    ret = X509_STORE_get_by_subject(&ctx, X509_LU_CRL, name, obj);
    X509_STORE_CTX_cleanup(&ctx);
    return ret;
}

/* -------------------------------------------------------------------------
 Լ̸ : bio_dump_rcb
      : 
 Է     : [in] BIO *bio
 Է     : [in] int cmd
 Է     : [in] const char *argp
 Է     : [in] int argi
 Է     : [in] long argl
 Է     : [in] long ret
      : long 
      : 
------------------------------------------------------------------------- */
long bio_dump_rcb(BIO *bio, 
				  int cmd, 
				  const char *argp, 
				  int argi, 
				  long argl, 
				  long ret)
{
	BIO *out, *outmem = NULL;
	unsigned char *pout;

	out = (BIO *)BIO_get_callback_arg(bio);

	if (out == NULL) return(ret);

#ifdef WIN32
	outmem = BIO_new(BIO_s_mem());
	if(NULL == outmem) 
		return ret;
#endif

	if (cmd == (BIO_CB_READ|BIO_CB_RETURN))	{
		BIO_printf(out,
			"read from %08X [%08lX] (%d bytes => %ld (0x%X))\n",	
			bio,argp,argi,ret,ret);
		BIO_dump(out,argp,(int)ret);
#ifdef WIN32
		BIO_printf(outmem,
			"read from %08X [%08lX] (%d bytes => %ld (0x%X))\n", 
			bio,argp,argi,ret,ret);
		BIO_dump(outmem,argp,(int)ret);
		BIO_write(outmem, "", 1);
		BIO_flush(outmem);
		BIO_get_mem_data(outmem, &pout);
		OutputDebugString(pout);
#endif
	}

#ifdef WIN32
	BIO_free(outmem);
#endif
	return(ret);
}


/* -------------------------------------------------------------------------
 Լ̸ : bio_dump_wcb
      : 
 Է     : [in] BIO *bio
 Է     : [in] int cmd
 Է     : [in] const char *argp
 Է     : [in] int argi
 Է     : [in] long argl
 Է     : [in] long ret
      : long 
      : 
------------------------------------------------------------------------- */
long bio_dump_wcb(BIO *bio, 
				  int cmd, 
				  const char *argp, 
				  int argi, 
				  long argl, 
				  long ret)
{
	BIO *out, *outmem = NULL;
	unsigned char *pout;

	out=(BIO *)BIO_get_callback_arg(bio);

	if (out == NULL) return(ret);

#ifdef WIN32
	outmem = BIO_new(BIO_s_mem());
	if(NULL == outmem) 
		return ret;
#endif

	if (cmd == (BIO_CB_WRITE|BIO_CB_RETURN)) {
		BIO_printf(out,
			"write to %08X [%08lX] (%d bytes => %ld (0x%X))\n", 
			bio,argp,argi,ret,ret);
		BIO_dump(out,argp,(int)ret);
#ifdef WIN32
		BIO_printf(outmem,
			"write to %08X [%08lX] (%d bytes => %ld (0x%X))\n",	
			bio,argp,argi,ret,ret);
		BIO_dump(outmem,argp,(int)ret);
		BIO_write(outmem, "", 1);
		BIO_flush(outmem);
		BIO_get_mem_data(outmem, &pout);
		OutputDebugString(pout);
#endif
	}
#ifdef WIN32
	BIO_free(outmem);
#endif
	return(ret);
}

void print_connect_info(BRM_SSL *bssl)
{
	/* const char *str; */
	X509 *peer;
	char buf[BUFSIZ];
	/* SSL_CIPHER *c; */

	peer = SSL_get_peer_certificate(bssl->ssl);
	BIO_printf(bssl->trcbio,"---\n");
	if(peer) {
		BIO_printf(bssl->trcbio,"PEER CERTIFICATE:\n");
		PEM_write_bio_X509(bssl->trcbio,peer);
		X509_NAME_oneline(X509_get_subject_name(peer), buf, BUFSIZ);
		BIO_printf(bssl->trcbio,"SUBJECT=%s\n",buf);
		X509_NAME_oneline(X509_get_issuer_name(peer), buf, BUFSIZ);
		BIO_printf(bssl->trcbio,"ISSUER=%s\n",buf);
		X509_free(peer);
	} else {
		BIO_printf(bssl->trcbio,"no peer certificate available\n");
	}
	BIO_printf(bssl->trcbio,
		"---\nSESSION=%s\n---\n", 
		(bssl->ssl->hit)?"Reused Session":"New Session");
	/*
	c = SSL_get_current_cipher(bssl->ssl);
	BIO_printf(bssl->trcbio,"PROTOCOL=%s\n", SSL_CIPHER_get_version(c));
	if(SSL_get_shared_ciphers(bssl->ssl, buf, BUFSIZ))
		BIO_printf(bssl->trcbio,"SHARED CIPHERS=%s\n", buf);
	str = SSL_CIPHER_get_name(c);
	BIO_printf(bssl->trcbio,"CIPHER=%s\n",(str != NULL)?str:"(NONE)");
	if(bssl->ssl->hit) 
		BIO_printf(bssl->trcbio, "Reused session-id\n");
	*/
	SSL_SESSION_print(bssl->trcbio, SSL_get_session(bssl->ssl));
	BIO_printf(bssl->trcbio,"---\n");

	if(SSL_ctrl(bssl->ssl, SSL_CTRL_GET_FLAGS, 0, NULL) 
		& TLS1_FLAGS_TLS_PADDING_BUG)
		BIO_printf(bssl->trcbio,
		"---\nPeer has incorrect TLSv1 block padding\n---\n");
}


int SSL_CTX_use_PrivateKey_file_ex(SSL_CTX *ctx, 
								   const char *file,  
								   int type, 
								   char *passwd)
{
	int ret=0;
	BIO *in;
	EVP_PKEY *pkey=NULL;

	in = BIO_new(BIO_s_file_internal());
	if(in == NULL)	{
		SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE,ERR_R_BUF_LIB);
		goto end;
	}

	if (BIO_read_filename(in,file) <= 0) {
		SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE,ERR_R_SYS_LIB);
		goto end;
	}
	if(type == SSL_FILETYPE_PEM)
		pkey = PEM_read_bio_PrivateKey(in,NULL, NULL, passwd);
	else if(type == SSL_FILETYPE_ASN1) 
		pkey = d2i_PKCS8PrivateKey_bio(in, NULL, NULL, passwd);
	
	if (pkey == NULL) {
		SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE, ERR_R_PEM_LIB);
		goto end;
	}
	ret = SSL_CTX_use_PrivateKey(ctx,pkey);
	EVP_PKEY_free(pkey);
end:
	if (in != NULL) BIO_free(in);
	return(ret);
}



/* -------------------------------------------------------------------------
 Լ̸ : chk_protocol_option
      : SSLProtocol ɼ Ѵ.
			line  Էµ ڿ which Էµ ڿ ȮϿ 
			wv ɼ  proto  ȮѴ. 
 Է     : [in] char *line - SSLProtocol  ڿ 
 Է     : [in] char *which - ɼ ڿ 
 Է     : [in] int wv - ɼ  
 Է     : [in] int *proto -   
      : int 
      : 
------------------------------------------------------------------------- */
int chk_protocol_option(char *line, char *which, int wv, int *proto)
{
	int ret = BRM_OK;
	char *p, *s = NULL, *t = NULL;
	
	/* capitalize string input */
	p = s = OPENSSL_malloc(strlen(line) + 1);
	if(!s) { 
		ret = BRM_ERR_OUTOF_MEMORY; 
		goto err; 
	}
	strcpy(s, line);
	while(*p) *p++ = (char)toupper(*p);

	p = t = (char *)OPENSSL_malloc(strlen(which) + 1);
	if(!t) { 
		ret = BRM_ERR_OUTOF_MEMORY; 
		goto err; 
	}
	strcpy(t, which);
	while(*p) *p++ = (char)toupper(*p);

	p = strstr(s, t);
	if(p) {
		if(p == s) {
			*proto |= wv;
		} else {
			if(*(--p) == '-') *proto &= ~wv;
			else if(*p == '+') *proto |= wv;
			else *proto |= wv;
		}
	}

err:
	if(s) OPENSSL_free(s);
	if(t) OPENSSL_free(t);
	return ret;
}



