/* blssl/brm_util.c */
/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).  
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its 
 *    contributors may not be used to endorse or promote products 
 *    derived from this software without specific prior written 
 *    permission. 
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */


#include <string.h>
#include <jni.h>

#include "brm_ssl.h"


/* -------------------------------------------------------------------------
 Լ̸ : get_jstr
      : ڹ Ŭ   ɹ  д´.
 Է     : [in] JNIEnv *env
 Է     : [in] jobject obj
 Է     : [in] const char *fname
 Է     : [in] char **value
      : int > 0 - б , ޸ Ȯ Ʈ ũ  
				< 0 - б , ޸ Ȯ  Ʈ ũ   
				= 0 - ڹ Ŭ  ã  ų, 
					  ǹִ   
------------------------------------------------------------------------- */
int get_jstr(JNIEnv *env, jobject obj, const char *fname, char **value)
{
	jclass cls;
	jfieldID fid;
	jstring tmp;
	jboolean isCopy= JNI_FALSE;
	char *val;
	int ret = 0;

	*value = NULL;

	cls = (*env)->GetObjectClass(env, obj);
	fid = (*env)->GetFieldID(env, cls, fname, "Ljava/lang/String;");
	tmp = (*env)->GetObjectField(env, obj, fid);
	/* Ʈ null̸ GetObjectField ȣ⿡ null  */
	if(!tmp) return ret;

	val = (char *)(*env)->GetStringUTFChars(env, tmp, &isCopy);
	if(!val) return ret;
	/* java ڿ ""  1Ʈ 0 ڰ ϵ, 
	  value     */
	if(val[0]) { 
		ret = strlen(val) + 1;
		*value = (char *)OPENSSL_malloc(ret);
		if(!(*value)) return -ret;
		strcpy(*value, val);
	}
	if(isCopy == JNI_TRUE) 
		(*env)->ReleaseStringUTFChars(env, tmp, val);

	return ret;
}


void set_jstr(JNIEnv *env, jobject obj, const char *fname, const char *val)
{
	jclass cls;
	jfieldID fid;
	jstring tmp;
	
	cls = (*env)->GetObjectClass(env, obj);
	tmp = (*env)->NewStringUTF(env, val);
	fid = (*env)->GetFieldID(env, cls, fname, "Ljava/lang/String;");
	(*env)->SetObjectField(env, obj, fid, tmp);
}


void get_jint(JNIEnv *env, jobject obj, const char *fname, int *value)
{
	jclass cls;
	jfieldID fid;
	
	*value = 0;
	cls = (*env)->GetObjectClass(env, obj);
	fid = (*env)->GetFieldID(env, cls, fname, "I");
	*value = (*env)->GetIntField(env, obj, fid);
}


void set_jint(JNIEnv *env, jobject obj, const char *fname, int val)
{
	jclass cls;
	jfieldID fid;
	
	cls = (*env)->GetObjectClass(env, obj);
	fid = (*env)->GetFieldID(env, cls, fname, "I");
	(*env)->SetIntField(env, obj, fid, (jint)val);
}

void get_jlong(JNIEnv *env, jobject obj, const char *fname, jlong *value)
{
	jclass cls;
	jfieldID fid;
	
	*value = 0;
	cls = (*env)->GetObjectClass(env, obj);
	fid = (*env)->GetFieldID(env, cls, fname, "J");
	*value = (*env)->GetIntField(env, obj, fid);
}


void set_jlong(JNIEnv *env, jobject obj, const char *fname, jlong val)
{
	jclass cls;
	jfieldID fid;
	
	cls = (*env)->GetObjectClass(env, obj);
	fid = (*env)->GetFieldID(env, cls, fname, "J");
	(*env)->SetIntField(env, obj, fid, (jint)val);
}

void raise_exception(JNIEnv *env, BIO *bio)
{
	jclass cls;
	char *buf = NULL;
	int len;

	if(bio) {
		len = BIO_ctrl_pending(bio);
		if(len > 0) {
			buf = (char *)OPENSSL_malloc(len + 1);
			if(buf) {
				BIO_read(bio, buf, len);
				buf[len] = 0;
			}
		} 
	}
	/* (*env)->ExceptionDescribe(env); */
	/* (*env)->ExceptionClear(env); */
	cls = (*env)->FindClass(env, "com/brm/net/ssl/SSLException");
	if (cls == 0) 
		/* Unable to find the new exception class, give up. */
		return; 
	(*env)->ThrowNew(env, cls, buf);
	OPENSSL_free(buf);
	return;
}



