/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */


/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */

import java.io.*;
/* the modified part to the plain socket application {{ */
import java.net.*;
import com.brm.net.ssl.*;
/* }} */

public class SSLClient
{
	public String readLine(BufferedInputStream input) throws IOException
	{
		byte buffer[] = new byte[2048];
		int next;
		int count = 0;
		while (true)
		{
			next = input.read();
			if (next < 0) {
				throw new IOException("The end of the stream has been reached");
			}
			if(next == '\n' || next == '\r') {
				buffer[count++] = (byte)'\n';
				break;
			}
			buffer[count++] = (byte) next;
			if (count >= buffer.length)
				throw new IOException ("input message too long");
		}
		return new String (buffer, 0, count);
    }

	public void run(String[] args)
	{
		boolean ssl = false;
		/* the added part to the plain socket application {{ */
		SSLGlobal gbl;
		SSLSession session;
		/* }} */
		/* the modified part to the plain socket application {{ */
		Socket psock = null;
		SSLSocket sock = null;
		/* }} */
		BufferedInputStream sin, cin;
		BufferedOutputStream sout, cout;
		String address;
		int port;
		String msg;

		System.out.println("usage : java SSLClient ssl address port conffile passwd");
		ssl = (Boolean.valueOf(args[0])).booleanValue();
		if(!ssl && args.length < 3) System.exit(1);
		if(ssl && args.length < 5) System.exit(2);
		System.out.print("inputs: [" + args[0] + "] [" + args[1] + "] [" + args[2] + "] ");
		if(ssl) System.out.print("[" + args[3] + "] [ " + args[4] + "]");
		System.out.println("");

		if(ssl) {
			System.out.println("SSLClient : try ssl connection...");
		} else {
			System.out.println("SSLClient : try plain socket connection...");
		}

		try {
			address = args[1];
			port = Integer.parseInt(args[2]);

			if(ssl) {
				gbl = new SSLGlobal(args[3], args[4]);
				System.out.println(gbl.toString());
				System.out.println("SSLClient : try connection to [" + address + ":" + port + "] ...");
				/* Demonstrate ssl connection resume {{ */
				// connect test at first
				sock = new SSLSocket(gbl, address, port);
				// save the ssl connection session information
				System.out.println("SSLClient : ssl connection and saving the ssl session succeed.");
				session = new SSLSession(sock);
				System.out.println("SSLClient : ssl connection closed, saving the ssl session.");
				sock.close();
				System.out.println("SSLClient : try ssl connection with the ssl session to [" + address + ":" + port + "] ...");
				/* }} */
				sock = new SSLSocket(gbl, session);
				System.out.println("SSLClient : ssl connection with the ssl session succeed.");
				sin = new BufferedInputStream(sock.getInputStream());
				sout = new BufferedOutputStream(sock.getOutputStream());
				cin = new BufferedInputStream(System.in);
				cout = new BufferedOutputStream(System.out);
				System.out.println("type message to send to server and press 'Q' and enter.");
			} else {
				System.out.println("SSLClient : try plain connection to [" + address + ":" + port + "]...");
				psock = new Socket(address, port);
				System.out.println("SSLClient : plain connection succeed.");
				sin = new BufferedInputStream(psock.getInputStream());
				sout = new BufferedOutputStream(psock.getOutputStream());
				cin = new BufferedInputStream(System.in);
				cout = new BufferedOutputStream(System.out);
				System.out.println("type message to send to server or (press 'Q' and enter) to quit.");
			}

			do {
				System.out.print("input message to sennd to server : ");
				msg = readLine(cin);
				// Java input system bug ?
				// Although I read string completely in the console buffered stream,
				// another character "line feed" remains in the input stream.
				// In JBuilder environment this demo however runs correctly, as it shoudl be.
				// So the console input buffer is cleared to run correctly all environment.
				while(cin.available() > 0) cin.read();
				sout.write(msg.getBytes());
				sout.flush();
				msg = readLine(sin);
				System.out.print("received echo message from server: ");
				cout.write(msg.getBytes());
				cout.flush();
			} while(msg.indexOf('Q') <  0);
			if(ssl) sock.close();
			else psock.close();

		} catch(Exception e) {
			e.printStackTrace();
		}
    }
	public static void main(String[] args)
	{
		SSLClient client = new SSLClient();
		client.run(args);
	}
}