/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */


/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */

import java.io.*;
/* the modified part to the plain socket application {{ */
import java.net.*;
import com.brm.net.ssl.*;
/* }} */
public class SSLServer
{
    public static void main(String[] args)
    {
		boolean ssl;
		/* the added part to the plain socket application {{ */
		SSLGlobal gbl;
		/* }} */
		/* the modified part to the plain socket application {{ */
		ServerSocket pssock = null;
		SSLServerSocket ssock = null;
		/* }} */
		SSLThread st;
		int port, max;

		System.out.println("usage : java SSLServer ssl bindport threads conffile passwd");
		ssl = (Boolean.valueOf(args[0])).booleanValue();
		if(!ssl && args.length < 3) System.exit(1);
		if(ssl && args.length < 5) System.exit(2);
		System.out.print("inputs: [" + args[0] + "] [" + args[1] + "] [" + args[2] + "] ");
		if(ssl) System.out.print("[" + args[3] + "] [ " + args[4] + "]");
		System.out.println("");

		if(ssl) {
			System.out.println("SSLServer : try ssl accept...");
		} else {
			System.out.println("SSLServer : try plain socket accept...");
		}

		try {
			port = Integer.parseInt(args[1]);
			if(ssl) {
				gbl = new SSLGlobal(args[3], args[4]);
				System.out.println(gbl.toString());
				ssock = new SSLServerSocket(gbl, port);
			} else {
				pssock = new ServerSocket(port);
			}
			max = Integer.parseInt(args[2]);
			System.out.println("SSLThread start...");
			for(int i = 0; i < max; i++) {
				if(ssl) st = new SSLThread(ssock);
				else st = new SSLThread(pssock);
				st.start();
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
    }
}