/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */


/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */


import java.io.*;
/* the modified part to the plain socket application {{ */
import java.net.*;
import com.brm.net.ssl.*;
/* }} */

public class SSLThread extends Thread
{
	boolean ssl = false;
	private static Object mutex = new Object();

	/* the modified part to the plain socket application {{ */
	private ServerSocket pssock = null;
	private	SSLServerSocket ssock = null;
	/* }} */

    public SSLThread(SSLServerSocket ssock) throws Exception
    {
		this.ssock = ssock;
		ssl = true;
    }

	public SSLThread(ServerSocket pssock) throws Exception
	{
		this.pssock = pssock;
		ssl = false;
    }

	public void run()
	{
	   System.out.println("SSLThrad (" + Thread.currentThread().getName() + "): run enter...");

	   while(true) {
		   try {
			   /* the modified part to the plain socket application {{ */
			   Socket psock = null;
			   SSLSocket sock = null;
			   /* }} */
			   System.out.println("SSLThrad (" + Thread.currentThread().getName() + "): run enter while loop.");
			   synchronized(mutex) {
				   if(ssl) {
					   sock = ssock.accept();
				   } else {
					   psock = pssock.accept();
				   }
			   }
			   System.out.println("SSLThread (" + Thread.currentThread().getName() + "): socket accepted.");

			   if(ssl) server(sock);
			   else server(psock);

		   } catch(Exception e) {
			   // to  prvent the ssl thread from dying, catch exceptions in the while loop.
			   e.printStackTrace();
		   }
	   }
    }

	public String readLine (BufferedInputStream input) throws IOException
	{
		byte buffer[] = new byte[2048];
		int next;
		int count = 0;
		while (true)
		{
			next = input.read();
			if (next < 0) {
				throw new IOException("the end of the stream has been reached.");
			}
			if(next == '\n' || next == '\r') {
				buffer[count++] = (byte)'\n';
				break;
			}
			buffer[count++] = (byte) next;
			if (count >= buffer.length)
				throw new IOException ("input message too long.");
		}
		return new String (buffer, 0, count);
	}

	/* the modified part to the plain socket application {{ */
	// private void server(Socket sock)
	private void server(SSLSocket sock)
	/* }} */
	{
		BufferedInputStream sin, cin;
		BufferedOutputStream sout, cout;

		try {
			sin = new BufferedInputStream(sock.getInputStream());
			sout = new BufferedOutputStream(sock.getOutputStream());
			cout = new BufferedOutputStream(System.out);
			// int cnt = 0;
			String msg;
			byte bmsg[];
			do {
				msg = readLine(sin);
				System.out.print("received message from client : ");
				cout.write(msg.getBytes());
				cout.flush();
				System.out.print("sending message to client    : ");
				cout.write(msg.getBytes());
				cout.flush();
				sout.write(msg.getBytes());
				sout.flush();
				// System.out.println("SSLThread (" + Thread.currentThread().getName() + "): " + ch + ":" + cnt++);
			} while(msg.indexOf('Q') < 0);
			sock.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	private void server(Socket sock)
	{
	 BufferedInputStream sin, cin;
	 BufferedOutputStream sout, cout;

	 try {
		 sin = new BufferedInputStream(sock.getInputStream());
		 sout = new BufferedOutputStream(sock.getOutputStream());
		 cout = new BufferedOutputStream(System.out);
		 // int cnt = 0;
		 String msg;
		 byte bmsg[];
		 do {
			 msg = readLine(sin);
			 System.out.print("received message from client : ");
			 cout.write(msg.getBytes());
			 cout.flush();
			 System.out.print("sending message to client    : ");
			 cout.write(msg.getBytes());
			 cout.flush();
			 sout.write(msg.getBytes());
			 sout.flush();
			 // System.out.println("SSLThread (" + Thread.currentThread().getName() + "): " + ch + ":" + cnt++);
		 } while(msg.indexOf('Q') < 0);
		 sock.close();
	 }
	 catch(Exception e)
	 {
		 e.printStackTrace();
	 }
	}
}