/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

package com.brm.net.ssl;

/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */

import java.net.Socket;
// sun ssl Ϸ ...
// import javax.net.ssl.*;

public class SSLGlobal
{
	private SSLConf conf;
	private long bctx = 0;
	// Common API
	/**
	 * bctx ڵ  .  Ʈ OpenSSL SSL_CTX
	 * Ѵ.
	 *
	 * @param conf SSLConf Ʈ
	 * @param SSLCertificateKeyPasswd Ű йȣ
	 * @return bctx ڵ
	 * @throws SSLException
	 */
	private native long nativeNew(SSLConf conf, String SSLCertificateKeyPasswd)
	throws SSLException;

	/**
	 * bctx ڵ ޸𸮿 Ѵ.
	 *
	 * @param bctx bctx ڵ
	 */
	private native void nativeFree(long bctx);

	/**
	 * ƹ  SSLGlobal Ʈ .
	 *    Է ʾǷ ⺻  Ѵ.
	 * ⺻   Ʒ . <br>
	 * <br>
	 * SSLConf { <br>
	 * 	   SSLHandShakeTimeout = [30 sec] <br>
	 * 	   SSLSessionCacheTimeout = [900 sec] <br>
	 *     SSLCACertificatePath = [null] <br>
	 *     SSLCARevocationPath = [null] <br>
	 *     SSLCARevocationAutoUpdate = [Off] <br>
	 *     SSLVerify = [none] <br>
	 *     SSLVerifyDepth = [0] <br>
	 *     SSLCertificateFile = [null] <br>
	 *     SSLCertificateKeyFile = [null] <br>
	 *     SSLProtocol = [SSLv3] <br>
	 *     SSLCipherSuite = [DES-CBC-SHA] <br>
	 *     SSLMessageTrace = [Off] <br>
	 * } <br>
	 *
	 * ַ    ʴ Ŭ̾Ʈ Ѵ.
	 *
	 * @throws SSLException
	 */
	public SSLGlobal() throws SSLException
	{
		conf = new SSLConf();
		conf.defaultConf();
		bctx = nativeNew(conf, null);
	}

	/**
	 *  ϰ Ű  ȣ ԷϿ SSLGlobal Ʈ .
	 *
	 * @param confFile   ġ
	 * @param passwd Ű ȣ(ȣ  Ű  "" Է)
	 * @throws SSLException
	 */
	public SSLGlobal(String confFile, String passwd) throws SSLException
	{
		conf = new SSLConf(confFile);
		bctx = nativeNew(conf, passwd);
	}

	/**
	 *  Ʈ Ű  ȣ ԷϿ SSLGlobal Ʈ .
	 *  ڴ   ʰ α׷   
	 * Ѵ.  ڸ ϴ    ̴. ⺻ ȣ 
	 * ϴ   α׷ ϱ  ⺻ ⺻  ڵ
	 *  SSLGlobal() ڸ ϴ   ϴ.
	 *
	 * @param conf SSLConf  Ʈ
	 * @param passwd Ű ȣ(ȣ  Ű  "" Է)
	 * @throws SSLException
	 */
	public SSLGlobal(SSLConf conf, String passwd) throws SSLException
	{
		bctx = nativeNew(conf, passwd);
	}

	public void finalize()
	{
		if(bctx != 0) {
			nativeFree(bctx);
			bctx = 0;
		}
	}

	public long getCTX()
	{
		return bctx;
	}

	public SSLConf getSSLConf()
	{
		return conf;
	}

	public String toString()
	{
		String ret;
		ret = "SSLGloval {\n" + conf.toString() + "\n}";
		return ret;
	}

	// Load the native routines.
	static
	{
		try
		{
			System.loadLibrary("blssl");
		}
		catch(Exception e)
		{
			System.out.println("blssl library not found");
			e.printStackTrace();
		}
	}
}