/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

package com.brm.net.ssl;

/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */

import java.io.*;
import java.net.Socket;

public class SSLInputStream extends InputStream
{
	// get the reference count of the Socket Object to prevent from being garbage collected;
	private SSLSocket sock;
	private InputStream ips;
	private long bssl;

	private native int nativeAvailable(long bssl);
    private native int nativeReadArray(long bssl, byte[] in, int inoff, int inlen, byte out[], int outoff, int outlen)
	throws SSLException;

	public SSLInputStream(SSLSocket sslsock, InputStream ips) throws IOException
	{
		this.sock = sslsock;
		this.ips = ips;
		bssl = this.sock.getSSL();
    }
/*
	public SSLInputStream(InputStream ips, long bssl) throws IOException
	{
		this.sock = null;
		this.ips = ips;
		this.bssl = bssl;
	}
*/
	public int read() throws IOException
	{
        int n;
		byte[] b = new byte[1];

		n = read(b);
		if(n < 0) return n;
		else return b[0];
	}

    public int read(byte b[]) throws IOException
	{
        return(read(b, 0, b.length));
    }

	/**
	 *
	 *  Լ  ۿ Ͱ  Ʈ  Ѵ.
	 * ׷ бۿ ѹƮ Ͱ ٸ ϴ Ư
	 *  InputStream.read() Լ  Ѵ.
	 *
	 * @param b -  ͸  迭
	 * @param off -  Ͱ  迭  ġ
	 * @param len -  Ͱ  迭 Ʈ ũ
	 * @return -  Ʈ 
	 * @throws IOException
	 */
	public int read(byte b[], int off, int len) throws IOException
	{
		int ret;
		int n;

		//  ۿ  Ͱ ϴ Ȯ
		n = nativeAvailable(bssl);
		if(n > 0) {
			if(n > len) n = len;
			try {
				ret = nativeReadArray(bssl, null, 0, 0, b, off, n);
			} catch(SSLException e) {
				e.printStackTrace();
				throw new IOException();
			}
		} else {
			//   ۿ Ͱ ٸ SocketInputStream ó
			// κ  б õ
			byte in[] = new byte[4096];
			n = ips.read(in, 0, in.length);
			if(n < 0) {
				// peer    IOException   ƴ϶
				// read()  -1 ´.  ȵǴ .
				// ׷ ⼭ SSLSeverSocket accept ó ܸ ߻Ű
				// ȵȴ. ֳϸ  read ޼尡 -1 ϵ
				// ο read ޼忡   ؾ Ѵ.
				// throw new IOException("the end of the stream has been reached");
				return n;
			}
			try {
				ret = nativeReadArray(bssl, in, 0, n, b, off, len);
			} catch(SSLException e) {
				e.printStackTrace();
				throw new IOException();
			}
		}
		return ret;
    }

    public long skip(long n) throws IOException
	{
		byte[] in = new byte[(int)n];

		return read(in, 0, in.length);
    }

  public int available() throws IOException
  {
	  byte in[] = new byte[4096];
	  int len;


	  len = ips.available();
	  if(len > 0) {
		  if(len > in.length) {
			 len = in.length;
			 ips.read(in, 0, len);
			 try {
				 nativeReadArray(bssl, in, 0, len, null, 0, 0);
			 } catch(SSLException e) {
				 e.printStackTrace();
				 throw new IOException();
			 }
		  }
	  }
	  return nativeAvailable(bssl);
  }

  public void close() throws IOException {
    ips.close();
  }

  public synchronized void mark(int readlimit)
  {
    // The mark method of InputStream does nothing.
	ips.mark(readlimit);
  }

  public synchronized void reset() throws IOException
  {
	// The method reset for class InputStream does nothing and always throws an IOException
    ips.reset();
  }

  public boolean markSupported()
  {
	// markSupported method of InputStream returns false.
    return(ips.markSupported());
  }
}