/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

package com.brm.net.ssl;

/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */

import java.io.*;
import java.net.Socket;

public class SSLOutputStream  extends OutputStream
{
	// get the reference count of the Socket Object to prevent from being garbage collected;
	private SSLSocket sock;
	private OutputStream ops;

	private native byte[] nativeWriteArray(long bssl, byte b[], int off, int len)
	throws SSLException;

    public SSLOutputStream(SSLSocket sslsock, OutputStream ops) throws IOException
	{
        this.sock = sslsock;
		this.ops = ops;
    }

	public void write(int b) throws IOException
    {
       byte[] in = new byte[1];

	   in[0] = (byte)b;
	   write(in);
    }

    public void write(byte b[]) throws IOException
	{
	    write(b, 0, b.length);
    }

    public void write(byte b[], int off, int len) throws IOException
	{
		int left, n, i;
		byte[] out;

		try {
			out = nativeWriteArray(sock.getSSL(), b, off, len);
			if(out != null) {
				ops.write(out);
			}
		} catch(SSLException e) {
			e.printStackTrace();
			throw new IOException();
		}
    }

    public void flush() throws IOException
	{
        ops.flush();
    }

  public void close() throws IOException
  {
    ops.close();
  }
}