/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

package com.brm.net.ssl;

/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */
import java.net.Socket;
import java.net.ServerSocket;
import java.net.SocketImplFactory;
import java.net.SocketException;
import java.net.InetAddress;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class SSLServerSocket
{
	private ServerSocket sock;
	private SSLGlobal gbl;

	/**
	 * ο SSL  ؽƮ .
	 *   SSL ؽƮ bctx    ޴´.
	 *
	 * @param bctx
	 * @return
	 * @throws SSLException
	 */
	private native long nativeNew(long bctx) throws SSLException;
	/**
	 * nativeNew   SSL ؽƮ Ѵ.
	 *
	 * @param bssl
	 */
	private native void nativeFree(long bssl);
	/**
	 * SSL  ϼǾ ȮѴ.
	 *
	 * @param bssl
	 * @return true SSL ϼ,
	 *         false SSL    
	 */
	private native boolean nativeAcceptFinished(long ssl);
	/**
	 * SSL  õ ޾ Ѵ.
	 *
	 * @param bssl SSL ؽƮ
	 * @param in Է Ʈ̿
	 * @param off Է Ʈ  offset
	 * @param len Է Ʈ ũ
	 * @return  Ʈ
	 *
	 * @throws SSLException
	 */
	private native byte[] nativeAccept(long ssl, byte[] in, int off, int len)
	throws SSLException;

	/**
	 * SSL   Ŭ   ´.
	 *
	 * @param gbl
	 * @throws SSLException gbl зͰ null Էµ 
	 */
	private void initialize(SSLGlobal gbl) throws SSLException
	{
		if(gbl == null)
			throw new SSLException("SSLGlobal parameter null input");

		this.gbl = gbl;
	}

	/**
	 * Ʒ ڵ ServerSocket Ŭ ڵ  տ
	 * gbl зͰ ߰   Ѵ.
	 * gbl з  Ŭ   ο   .
	 *
	 * @param gbl SSL  ؽƮ
	 * @param port
	 * @throws IOException
	 * @throws SSLException
	 */
	public SSLServerSocket(SSLGlobal gbl, int port)
	throws IOException, SSLException
	{
		initialize(gbl);
		sock = new ServerSocket(port);
	}

	public SSLServerSocket(SSLGlobal gbl, int port, int backlog)
	throws IOException, SSLException
	{
		initialize(gbl);
		sock = new ServerSocket(port, backlog);
	}

	public SSLServerSocket(SSLGlobal gbl, int port, int backlog, InetAddress bindAddr)
	throws IOException, SSLException
	{
		initialize(gbl);
		sock = new ServerSocket(port, backlog, bindAddr);
	}

	/**
	 * Ҹ
	 *
	 */
	public void finalize()
	{
		/*
		SSLServerSocket ڽ  SSL  SSLSocket Ʈ Ѱְ
		ڽ SSL   Ƿ ̰ SSL   ʿ䰡 .
		̰ SSLSocket Ҹڿ ٸ̴.
		*/
	}

	/**
	 * ServerSocket Ŭ accept ޼忡 ϴ SSLServerSocket Ŭ
	 * accept ޼̴.  ޼ SSL  ޾  ϰ   
	 * ο SSLSocet ȣ ʿ  ش.
	 *
	 * @return SSL  ϼ  SSLSocket Ʈ
	 * @throws IOException Socket Ŭ 
	 * @throws SSLException SSL   ߻ϴ 
	 */
	public SSLSocket accept() throws IOException, SSLException
	{
		SSLSocket ssock;
		Socket sock;
		long bssl;
		DataInputStream ips;
	    DataOutputStream ops;
		int timeout;

		sock = this.sock.accept();
		bssl = nativeNew(gbl.getCTX());
		ssock = new SSLSocket(bssl, sock);
		// save original timeout
		timeout = ssock.getSoTimeout();
		ssock.setSoTimeout(gbl.getSSLConf().SSLHandShakeTimeout * 1000);
		ops = new DataOutputStream(sock.getOutputStream());
		ips = new DataInputStream(sock.getInputStream());
/*
		 read() ޼  Ư
			    socket ۿ   ٸ timeout 
			timeout Ǹ
		java.net.SocketTimeoutException: Read timed out
			at java.net.SocketInputStream.socketRead0(Native Method)
			at java.net.SocketInputStream.read(SocketInputStream.java:116)
			at java.io.DataInputStream.read(DataInputStream.java:167)
			at com.brm.net.ssl.SSLServerSocket.accept(SSLServerSocket.java:100)
			at SSL_Server.run(SSL_Server.java:34)
			at SSL_Server.main(SSL_Server.java:59)
		׷Ƿ ݵ timeout    Ѵ.
*/
		byte ret[], in[] = new byte[4096];
		int len;
		do {
			len = ips.read(in, 0, in.length);
			if(len == -1) {
				// peer    IOException   ƴ϶
				// read()  -1 ´.  ȵǴ .
				throw new IOException("the end of the stream has been reached");
			}
			ret = nativeAccept(bssl, in, 0, len);
			if(ret != null) {
				ops.write(ret);
				ops.flush();
			}
		} while (!nativeAcceptFinished(bssl));

		ssock.setSoTimeout(timeout);
		return ssock;
	}

	public InetAddress getInetAddress()
	{
	    return sock.getInetAddress();
    }

    public int getLocalPort()
	{
	    return sock.getLocalPort();
    }

    public void close() throws IOException
	{
	    sock.close();
    }

    public synchronized void setSoTimeout(int timeout) throws SocketException
	{
	    sock.setSoTimeout(timeout);
    }

    public synchronized int getSoTimeout() throws IOException
	{
	    return sock.getSoTimeout();
    }

    public String toString()
	{
	    return ("SSL" + sock.toString());
    }

    public static synchronized void setSocketFactory(SocketImplFactory fac) throws IOException
	{
		ServerSocket.setSocketFactory(fac);
	}
}
