/* ====================================================================
 * Copyright (C) 2002 Jung-Ho Cha (hinunbi@users.kldp.net).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the "Barunmo" nor the names of its
 *    contributors may not be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *    For written permission, please contact hinunbi@users.kldp.net.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes software developed by the OpenSSL Project
 * for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

package com.brm.net.ssl;

/**
 * <p>Title: Barunmo Java SSL Socket Class</p>
 * <p>Description: SSL Socket Class by using OpenSSL</p>
 * <p>Copyright: Copyright (c) 2002  Jung-Ho Cha</p>
 * <p>Company: Barunmo</p>
 * @author Jung-Ho Cha
 * @version 1.0.0
 */

import java.net.Socket;
import java.net.SocketImpl;
import java.net.SocketImplFactory;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.SocketException;

public class SSLSocket
{
	//private static SocketImplFactory factory;
	private Socket sock = null;
	private long bssl = 0;
	private SSLGlobal gbl;

	/**
	 * ο SSL  ؽƮ .
	 *   SSL ؽƮ bctx    ޴´.
	 *
	 * @param bctx
	 * @return
	 * @throws SSLException
	 */
	private native long nativeNew(long bctx) throws SSLException;
	/**
	 * nativeNew   SSL ؽƮ Ѵ.
	 *
	 * @param bssl
	 */
	private native void nativeFree(long bssl);
	/**
	 * SSL  ϼǾ ȮѴ.
	 *
	 * @param bssl
	 */
	private native boolean nativeConnectFinished(long bssl);
	/**
	 * SSL  ݴ´.
	 *
	 * @param bssl
	 */
	private native void nativeShutdown(long bssl);



	private native void nativeAsignSession(long bssl, byte sess[], int len)
	throws SSLException;

	 /**
	  * SSL  õѴ.
	  *
	  * @param bssl - SSL ؽƮ
	  * @param in - Է Ʈ̿
	  * @param off - Է Ʈ  offset
	  * @param len - Է Ʈ ũ
	  * @return -  Ʈ
	  *
	  * @throws SSLException
	  */
	private	native byte[] nativeConnect(long bssl, byte[] in, int off, int len)
	throws SSLException;

	/**
	 * ο SSL ؽƮ .
	 *
	 * @param gbl - SSL  ϴ  
	 *
	 * @throws SSLException
	 */
	private void initialize(SSLGlobal gbl) throws SSLException
	{
		if(gbl == null)
			gbl = new SSLGlobal();

		this.gbl = gbl;
		bssl = nativeNew(gbl.getCTX());
	}

	public long getSSL()
	{
		return bssl;
	}
	/**
	 *  accept     SSLSocket   
	 *  ڴ  Ѵ. Ŭ̾Ʈ  ڸ ϴ
	 * 찡 Ͼ  ̴.
	 *
	 * @param bssl - SSL ڵ彦ũ  SSL ؽƮ
	 * @param sock -  socket Ʈ
	 *
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws SSLException
	 */
	public SSLSocket(long bssl, Socket sock)
	throws UnknownHostException, IOException, SSLException
	{
		this.sock = sock;
		this.bssl = bssl;
	}

	public SSLSocket(SSLGlobal gbl, SSLSession session)
	throws UnknownHostException, IOException, SSLException
	{
		byte sess[];
		if(session == null) {
			throw new SSLException("SSLSession object null");
		}
		sock = new Socket(session.getInetAddress(), session.getPort());
		initialize(gbl);
		sess = session.getSession();
		if(sess == null) {
			throw new SSLException("SSLSession object has no session info");
		}
		nativeAsignSession(bssl, sess, sess.length);
		SSLConnect();
	}

	/**
	 * Ʒ ڵ Socket Ŭ ڿ   зιͰ
	 * SSLClobal зͶ  ̿ܿ   .
	 * Socket Ŭ   ڸ ϹǷ ̹ ߵ
	 * ø̼ǿ SSL  ϱ  ߴ.
	 *
	 * @param gbl - SSLGlobal Ʈ
	 * @param host -  Ʒ Socket  зͿ 
	 * @param port
	 *
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws SSLException
	 */

	public SSLSocket(SSLGlobal gbl, String host, int port)
	throws UnknownHostException, IOException, SSLException
    {
		sock = new Socket(host, port);
		initialize(gbl);
		SSLConnect();
    }

    public SSLSocket(SSLGlobal gbl, InetAddress address, int port)
	throws IOException, SSLException
	{
	    sock = new Socket(address, port);
		initialize(gbl);
		SSLConnect();
    }

    public SSLSocket(SSLGlobal gbl, String host, int port, InetAddress localAddr, int localPort)
	throws IOException, SSLException
	{
	    sock = new Socket(host, port, localAddr, localPort);
		initialize(gbl);
		SSLConnect();
    }

    public SSLSocket(SSLGlobal gbl, InetAddress address, int port, InetAddress localAddr, int localPort)
	throws IOException, SSLException
	{
		sock = new Socket(address, port, localAddr, localPort);
		initialize(gbl);
		SSLConnect();
    }

    public SSLSocket(SSLGlobal gbl, String host, int port, boolean stream)
	throws IOException, SocketException
	{
	    throw new SocketException("not supported constructor");
    }

    public SSLSocket(SSLGlobal gbl, InetAddress host, int port, boolean stream)
	throws IOException, SocketException
	{
		throw new SocketException("not supported constructor");
    }

	/**
	 * bssl  jni ο  ؽƮ ϱ  ̹Ƿ
	 *  Ŭ Ҹ κп ޸𸮸 Ѵ.
	 */
	public void finalize()
	{
		if(bssl != 0) {
			nativeFree(bssl);
			bssl = 0;
		}
	}

	/**
	 * SSL Ѵ.
	 *  ޼ Socket Ŭó ڰ    ߴ.
	 *  4k   Ͽ ڵ彦ŷ Ѵ.  
	 * ũ Ʃ      ̴.
	 *
	 * @throws IOException
	 * @throws SSLException
	 */
	private void SSLConnect() throws IOException, SSLException
	{
		DataInputStream ips;
	    DataOutputStream ops;
		int timeout;

		// save original timeout
		timeout = sock.getSoTimeout();
		sock.setSoTimeout(gbl.getSSLConf().SSLHandShakeTimeout * 1000);
		ips = new DataInputStream(sock.getInputStream());
		ops = new DataOutputStream(sock.getOutputStream());

		byte ret[], in[] = new byte[4096];
		int len;
		// handshake start ...
		ret = nativeConnect(bssl, null, 0, 0);
		if(ret != null) {
			ops.write(ret);
			ops.flush();
		}
		do {
		    len = ips.read(in, 0, in.length);
			if(len == -1) {
				// peer    IOException   ƴ϶
				// read()  -1 ´.  ȵǴ .
				throw new IOException("the end of the stream has been reached");
			}
			ret = nativeConnect(bssl, in, 0, len);
			if(ret != null) {
				ops.write(ret);
				ops.flush();
			}
		} while (!nativeConnectFinished(bssl));
		//    Ѵ.
		// session = nativeGetSession(bssl);
		sock.setSoTimeout(timeout);
    }

	/**
	 * ̰ʹ Socket Ŭ ޼  ޼带 Ѵ.
	 *
	 * @return
	 */
	public InetAddress getInetAddress()
	{
	    return sock.getInetAddress();
    }

    public InetAddress getLocalAddress()
	{
		return sock.getLocalAddress();
    }

    public int getPort()
	{
	    return sock.getPort();
    }

    public int getLocalPort()
	{
	    return sock.getLocalPort();
    }

    public InputStream getInputStream() throws IOException
	{
		return new SSLInputStream(this, sock.getInputStream());
    }

    public OutputStream getOutputStream() throws IOException
	{
		return new SSLOutputStream(this, sock.getOutputStream());
    }

    public void setTcpNoDelay(boolean on) throws SocketException
	{
		sock.setTcpNoDelay(on);
    }

    public boolean getTcpNoDelay() throws SocketException
	{
	    return sock.getTcpNoDelay();
    }

    public void setSoLinger(boolean on, int linger) throws SocketException
	{
		sock.setSoLinger(on, linger);
    }

    public int getSoLinger() throws SocketException
	{
	    return sock.getSoLinger();
    }

    public synchronized void setSoTimeout(int timeout) throws SocketException
	{
		sock.setSoTimeout(timeout);
    }

    public synchronized int getSoTimeout() throws SocketException
	{
		return sock.getSoTimeout();
    }

    public synchronized void setSendBufferSize(int size)
    throws SocketException
	{
		sock.setSendBufferSize(size);
    }

    public synchronized int getSendBufferSize() throws SocketException
	{
		return sock.getSendBufferSize();
    }

    public synchronized void setReceiveBufferSize(int size)
    throws SocketException
	{
	    sock.setReceiveBufferSize(size);
    }

    public synchronized int getReceiveBufferSize()
    throws SocketException
	{
	    return sock.getReceiveBufferSize();
    }

    public void setKeepAlive(boolean on) throws SocketException
	{
		sock.setKeepAlive(on);
    }

    public boolean getKeepAlive() throws SocketException
	{
	    return sock.getKeepAlive();
    }

    public synchronized void close() throws IOException
	{
	    /*
		̰ SSL connecton       غ ̴.
		 ׽Ʈ  ƾ  ̴.
		     ʱ Ͽ.
		׸ SSL ݴ   ̰ ݾƵ  Ҹڿ  Ͽ.
		*/
		sock.close();
		nativeShutdown(bssl);
    }

    public void shutdownInput() throws IOException
    {
		sock.shutdownInput();
    }

    public void shutdownOutput() throws IOException
    {
		sock.shutdownOutput();
    }

    public String toString()
	{
		return ("SSL" + sock.toString());
	}
}


